% Preliminary
clear variables
close all

%% Uniform Random Model
load '.\Replication_IntermediateData\data_temp_all_64.mat'    

dd_coll         = individ_data(:,2:end); % pull the choice indicators for individuals.
lambda_coll     = zeros(size(individ_data,1),1);
pp_coll         = zeros(size(individ_data,1),5);
for ii=1:size(individ_data,1)
    temp_id         = individ_data(ii,1);
    lambda_coll(ii) = types_prices_mus(temp_id,end);
    pp_coll(ii,:)   = types_prices_mus(temp_id,1:5);   
end

N_menu_coll         =   length(deduc_list_coll);
Num_Obs             =   length(lambda_coll);

NrL        =  0;
NrH        =  1;
r_bar      =  .03;
Nods       =  121;
    
TT_r       =  linspace(NrL,NrH,Nods);
TT_r_mid   =  .5*(TT_r(1:end-1)+TT_r(2:end));

%% MLE Routine
EU_order_coll     = zeros(Num_Obs,Nods-1);
for tt=1:Nods-1
    EU_order_coll(:,tt)  = compute_l_h_order(TT_r_mid(tt)*r_bar,lambda_coll,pp_coll,deduc_list_coll,dd_coll,N_menu_coll);
end

Q_coll  = zeros(N_menu_coll,N_menu_coll);
for kk = 1:N_menu_coll
    for mm = 1:N_menu_coll
       if mm <= N_menu_coll -  kk + 1
           Q_coll(kk,mm) = nchoosek(N_menu_coll - mm, kk-1)/nchoosek(N_menu_coll , kk);
       end
    end
end

Qord_coll = zeros(N_menu_coll,Num_Obs,Nods-1);
for tt=1:Nods-1
    for nn = 1:Num_Obs
        Qord_coll(:,nn,tt) = Q_coll(:, EU_order_coll(nn,tt));
    end
end

mixture =1;
param_low  = [ .1    10           1e-6*ones(1,N_menu_coll)].';
param_high = [15    250       (1-1e-6)*ones(1,N_menu_coll-2) 1e-6*ones(1,2)].';

A_eq       = [0 0 ones(1,N_menu_coll)];
b_eq       = 1;
ind_att    = 3;

% MultiStart
param_0   = .5*(param_low+param_high);
D         = length(param_0);
Num_start = 20;
rs        = RandomStartPointSet('NumStartPoints',Num_start);
problem   = createOptimProblem('fmincon','x0',param_0,'lb',param_low,'ub',param_high,'Aeq',A_eq,'beq',b_eq);
points    = list(rs,problem);
options   = optimoptions(@fmincon,'TolFun',1e-7,'TolX',1e-7,'Algorithm','active-set','Display','iter');

points  = points.';
points(ind_att:end,:) = points(ind_att:end,:)./sum( points(ind_att:end,:));

param       = zeros(length(param_0),Num_start);
LLL         = zeros(Num_start,1);
flag_feas   = zeros(Num_start,1);
obj_fun = @(theta)Simulated_LLL_RCL(theta,TT_r,Nods-1,Num_Obs,Qord_coll);

t1 = tic;
parfor ms = 1:Num_start
    [param(:,ms),LLL(ms),flag_feas(ms)]   = fmincon(obj_fun,points(:,ms),[],[],A_eq,b_eq,param_low,param_high,[],options);
end
time_MLE = toc(t1);
disp(param)
[LLL_coll,index_out] = min(LLL);
param_coll           = param(:,index_out);
[~,LLL_coll_vec]     = obj_fun(param_coll);
 save('.\Replication_Results\comparator_UR_full_2_and_above','param_coll','LLL_coll','LLL_coll_vec') 

%% Bootstrap: UR
boot_reps       = 1000;
rng             = 82720;

bb_samp_ind     = randi(Num_Obs,Num_Obs,boot_reps);
param_coll_boot = zeros(7,boot_reps);
LLL_coll_boot   = zeros(1,boot_reps);
points_start    = param_coll;
parfor bb=1:boot_reps
    bb
    bb_ind      = bb_samp_ind(:,bb);
    Qord_coll_bb = Qord_coll(:,bb_ind,:);

    options   = optimoptions(@fmincon,'TolFun',1e-7,'TolX',1e-7,'Algorithm','active-set','Display','iter');
    obj_fun = @(theta)Simulated_LLL_RCL(theta,TT_r,Nods-1,Num_Obs,Qord_coll_bb);

    t1 = tic;
    [param,LLL,flag_feas]   = fmincon(obj_fun,points_start,[],[],A_eq,b_eq,param_low,param_high,[],options);
    time_MLE = toc(t1);
    disp(param)
    [LLL_coll,index_out] = min(LLL);
    param_coll           = param(:,index_out);
    [~,LLL_coll_vec]     = obj_fun(param_coll);

    param_coll_boot(:,bb) = param_coll;
    LLL_coll_boot(bb) = LLL_coll;
end

mean_boot       = r_bar.*param_coll_boot(1,:)./(param_coll_boot(1,:)+param_coll_boot(2,:));
var_boot        = r_bar.^2.*param_coll_boot(1,:).*param_coll_boot(2,:)./((param_coll_boot(1,:)+param_coll_boot(2,:)).^2.*(1+param_coll_boot(1,:)+param_coll_boot(2,:)));
quantiles_p = [.25 .75];
Y_inv_UR_boot = zeros(1000,length(quantiles_p));
for ii=1:boot_reps
   theta_temp = param_coll_boot(1:2,ii);
   Y_inv_UR_boot(ii,:) = betainv(quantiles_p,theta_temp(1),theta_temp(2));  
  
end
beta_invY_UR_boot = r_bar*Y_inv_UR_boot;
save('.\Replication_Results\comparator_UR_full_2_and_above_boot','param_coll_boot','LLL_coll_boot','mean_boot','var_boot','Y_inv_UR_boot','beta_invY_UR_boot') 


%% ASR Model:
Num_Sets    = 1+N_menu_coll+size(nchoosek(1:N_menu_coll,N_menu_coll-2),1);
Sets        = zeros(Num_Sets,N_menu_coll);
Sets(1,:)   = 1;
temp1       = nchoosek(1:N_menu_coll,N_menu_coll-1);
for jj=1:size(temp1,1)
Sets(1+jj,temp1(jj,:))  =1;
end
temp2       = nchoosek(1:N_menu_coll,N_menu_coll-2);
for jj=1:size(temp2,1)
Sets(1+size(temp1,1)+jj,temp2(jj,:))  =1;
end

EU_drop_coll     = zeros(Nods-1,Num_Obs,N_menu_coll);
EU_keep_coll     = dd_coll==1;
EU_drop_coll_vec = zeros(Nods-1,Num_Obs,N_menu_coll);
EU_stay_coll     = zeros(Nods-1,Num_Obs,Num_Sets);
parfor tt=1:Nods-1
    rr      = TT_r_mid(tt);
    temp                     = compute_l_h_val(rr*r_bar,lambda_coll,pp_coll,deduc_list_coll,dd_coll,N_menu_coll,'CARA');
    EU_stay_coll (tt,:,:)    = (temp*Sets'==0 & EU_keep_coll*Sets'==1);
end
EU_stay_coll    = permute(EU_stay_coll,[2,3,1]);

param_low            = 1e-8*ones(1,N_menu_coll);
param_high           =   .7*ones(1,N_menu_coll);
param_low  = [1e-6 1e-6              param_low];
param_high = [2   10                 param_high];

% MultiStart
param_0   = .5*(param_low+param_high);
Num_start = 20;
rs        = RandomStartPointSet('NumStartPoints',Num_start);
problem   = createOptimProblem('fmincon','x0',param_0,'lb',param_low,'ub',param_high);
points    = list(rs,problem);
options   = optimoptions(@fmincon,'TolFun',1e-7,'TolX',1e-7,'Algorithm','active-set','Display','iter','MaxFunctionEvaluations',500*(Num_Sets+2));
param   = zeros(Num_start,length(param_0));
LLL     = zeros(Num_start,1);

t1 = tic;
parfor ms = 1:Num_start
    [param(ms,:),LLL(ms)]   = fmincon(@Simulated_LLL_param_bcmt,points(ms,:),[],[],[],[],param_low,param_high,[],options,TT_r,Nods-1,Num_Obs,EU_stay_coll,Sets);
end
time_MLE = toc(t1);

disp(param)
[LLL_coll,index_out] = min(LLL);
param_coll           = param(index_out,:);
[~,LLL_coll_vec]     = Simulated_LLL_param_bcmt(param_coll,TT_r,Nods-1,Num_Obs,EU_stay_coll,Sets);
save( '.\Replication_Results\comparator_ASR_full_2_and_above','param_coll','LLL_coll','LLL_coll_vec') 

%% Bootstrap: ASR
bb_samp_ind     = randi(Num_Obs,Num_Obs,boot_reps);
param_coll_boot = zeros(7,boot_reps);
LLL_coll_boot   = zeros(1,boot_reps);
points_start    = param_coll;
options         = optimoptions(@fmincon,'TolFun',1e-7,'TolX',1e-7,'Algorithm','active-set','Display','iter','MaxFunctionEvaluations',500*(Num_Sets+2));
parfor bb=1:boot_reps
    bb_ind      = bb_samp_ind(:,bb);
    EU_stay_coll_bb = EU_stay_coll(bb_ind,:,:);
 
    t1                      = tic;
        [param,LLL]         = fmincon(@Simulated_LLL_param_bcmt,points_start,[],[],[],[],param_low,param_high,[],options,TT_r,Nods-1,Num_Obs,EU_stay_coll_bb,Sets);
    time_MLE                = toc(t1);

    disp(param)
    [LLL_coll,index_out]    = min(LLL);
    param_coll              = param(index_out,:);
    [~,LLL_coll_vec]        = Simulated_LLL_param_bcmt(param_coll,TT_r,Nods-1,Num_Obs,EU_stay_coll,Sets);

    param_coll_boot(:,bb)   = param_coll;
    LLL_coll_boot(bb)       = LLL_coll;
end

mean_boot       = r_bar.*param_coll_boot(1,:)./(param_coll_boot(1,:)+param_coll_boot(2,:));
var_boot        = r_bar.^2.*param_coll_boot(1,:).*param_coll_boot(2,:)./((param_coll_boot(1,:)+param_coll_boot(2,:)).^2.*(1+param_coll_boot(1,:)+param_coll_boot(2,:)));
quantiles_p = [.25 .75];
Y_inv_ASR_boot = zeros(1000,length(quantiles_p));
for ii=1:1000
   theta_temp = param_coll_boot(1:2,ii);
   Y_inv_ASR_boot(ii,:) = betainv(quantiles_p,theta_temp(1),theta_temp(2));  
end
beta_invY_ASR_boot = r_bar*Y_inv_ASR_boot;
save( '.\Replication_Results\comparator_ASR_full_2_and_above_boot','param_coll_boot','LLL_coll_boot','mean_boot','var_boot','Y_inv_ASR_boot','beta_invY_ASR_boot') 


