clear variables
close all

load .\Data\Replication_Data
r_bar                   = .03; 
covariate_group         = 'all';
dim_p                   = 2;
seed                    = 0; 
B                       = 1000; 
bin_thresh              = 100; 

loadfilename = ['.\Replication_IntermediateData\data_temp_',covariate_group,'_64.mat']; 
load(loadfilename)

d_sets  = [ 100  0; ...
            200  0; ...
            250  0; ...
            500  0; ...
            1000 0; ...
            100  200;...
            100  250;...
            100  500;...
            100  1000;...
            200  250;...
            200  500;...
            200  1000;...
            250  500;...
            250  1000;...
            500  1000];
                    
% Empirical Probabilities:
Sample_Size             = sum(types_counts_taus(:,1));
[f,f_keep,J1,phi_jt]    = moments_w( types_choice_counts,types_counts_taus,map_type_bin,d_sets,deduc_list_coll,bin_size,bin_thresh);
% Bootstrap
f_b     = zeros(J1,B); 
N_tb    = zeros(size(map_type_bin,1),B); 
parfor bb = 1:B
    bb
    stream=RandStream('mlfg6331_64','Seed',seed);
    RandStream.setGlobalStream(stream);
    stream.Substream = bb + B*10^3;

    ind_bb      = ceil(Sample_Size*rand(Sample_Size,1)); % draw sample
    ind_data_bb = individ_data(ind_bb,:); %#ok<PFBNS>
    
    types_choices_bb        = zeros(size(types_choice_counts));
    types_counts_taus_bb    = zeros(size(types_counts_taus));
    for tt=1:size(types_choices_bb,1)
        temp_bb = ind_data_bb(:,1)==tt;
        types_counts_taus_bb(tt,1) = sum(temp_bb);    
        types_choices_bb(tt,:) = sum(ind_data_bb(temp_bb==1,2:end),1);
        types_counts_taus_bb(tt,2) = types_counts_taus_bb(tt,1)/Sample_Size;
    end
    N_tb(:,bb) = types_counts_taus_bb(:,1);
    
    bin_size_bb = zeros(length(bin_size),1);
    for aa=1:length(bin_size)
        temp_bin_ind    = map_type_bin(map_type_bin(:,2)==aa,1); %#ok<PFBNS>
        bin_size_bb(aa)  = sum( types_counts_taus_bb(temp_bin_ind,1));   
    end
    f_temp      = moments_w(types_choices_bb,types_counts_taus_bb,map_type_bin,d_sets,deduc_list_coll,bin_size_bb,bin_thresh);
    f_b(:, bb)  = f_temp;
end

% Save File
savefile = ['.\Replication_IntermediateData\data_temp_Logit_empshares_',covariate_group,'_64_',num2str(dim_p),'.mat'];
save(savefile,'f','f_keep','phi_jt','f_b','N_tb','d_sets');


% Model Shares:
average_price = sum(types_prices_mus(:,1:5).*types_counts_taus(:,2));
average_price_diff =mean(diff(average_price));

std_ExVT1               = pi/sqrt(6);
Logit_Factor            = .5;
Logit_Sigma             = Logit_Factor*abs(average_price_diff)/std_ExVT1;
Nods                    = 121;  
NrL                     = 0;
NrH                     = 1;
TT_r                    = linspace(NrL,NrH,Nods);
MM_r                    = length(TT_r);
TT_r_expec              = [0 .5*(TT_r(1:end-1)+TT_r(2:end))];
Weighted_CE             = construct_Weighted_CE(types_prices_mus(:,6),types_prices_mus(:,1:5)-types_prices_mus(:,5),deduc_list_coll,TT_r_expec*r_bar,Logit_Sigma);
Logit_freq_by_type_and_r= zeros(length(types_prices_mus(:,6)),15,length(TT_r_expec));
for n_r = 1:length(TT_r_expec)
    parfor n_types = 1:length(types_prices_mus(:,6))
    [Logit_Singleton,Logit_Pairs] = logit_shares(squeeze(Weighted_CE(n_types,:,n_r)),5);
    temp = [Logit_Singleton Logit_Pairs];
    Logit_freq_by_type_and_r(n_types,:,n_r) = temp;
    end
end


savefile = ['.\Replication_IntermediateData\data_temp_Logit_modshares_',covariate_group,'_64_2_LogitFactor',num2str(Logit_Factor),'.mat'];
save(savefile,'Logit_freq_by_type_and_r','d_sets','TT_r','TT_r_expec','MM_r');


% AS Grids:
N_j         = size(f,1);
N_t         = size(map_type_bin,1);
d_sets  = [ 100  0; ...
            200  0; ...
            250  0; ...
            500  0; ...
            1000 0; ...
            100  200;...
            100  250;...
            100  500;...
            100  1000;...
            200  250;...
            200  500;...
            200  1000;...
            250  500;...
            250  1000;...
            500  1000];
N_d         = size(d_sets,1);
  


% Set parameters for Andrews & Shi test:
alpha       =.05;
eta         = 10^-6; % Adjustment param for Andrews and Shi, rec 10^-6;
kappa       = sqrt(.3*log(Sample_Size)); % GMS tuning parameter.
Bn_param    = sqrt(.4*log(Sample_Size)/log(log(Sample_Size))); % GMS param..

% Indicator of whether a given type is included in a given constraint
ind_type_t_in_j                     = zeros(N_t,N_j);
ind_type_t_in_j(:,end-N_d+1:end)    = 1;  
for tt=1:N_t
   temp_bin = map_type_bin(tt,2);
   ind_type_t_in_j(tt,(temp_bin-1)*N_d+1:temp_bin*N_d)=1;
end

tau_t       = types_counts_taus(:,2);
tau_tb      = N_tb./Sample_Size;
alpha_adj 	= alpha - eta; 



%% set the master grid 
mean_low        = 0;
mean_high       = 1;
var_low         = 0;
var_high        = 0.25;
grid_level      = 4;

N_master_mean   = 100*2^(grid_level-1);
N_master_var    =  25*2^(grid_level-1);
master_means    = linspace(mean_low,mean_high,N_master_mean+1);
master_vars     = linspace(var_low,var_high,N_master_var+1);
master_means    = master_means(2:end);
master_vars     = master_vars(2:end);
master_alpha    = (master_means.^3.*(1./master_means-1))'*(1./master_vars)-repmat(master_means',1,length(master_vars));
master_beta     = master_alpha.*repmat(((1-master_means)./master_means)',1,length(master_vars));

MASTER_BETA             = [master_alpha(:) master_beta(:)];
temp_mean               = repmat(master_means,1,N_master_var);
temp_var                = repmat(master_vars,N_master_mean,1);
MASTER_MEAN_VAR         = [temp_mean(:) temp_var(:)];
MASTER_pass             = zeros(N_master_mean*N_master_var,1);
MASTER_checked          = zeros(N_master_mean*N_master_var,1);

MASTER_Matrix_Pass      =  zeros(N_master_mean,N_master_var);
MASTER_Matrix_Checked   =  zeros(N_master_mean,N_master_var);
MASTER_Matrix_Check     =  zeros(N_master_mean,N_master_var);

GRID_MATRIX             = zeros(N_master_mean,N_master_var,grid_level);
mean_grid_index         = zeros(N_master_mean,grid_level);
var_grid_index          = zeros(N_master_var,grid_level);

for grid_refinement=1:grid_level
mean_grid_index((1:(2^grid_level/2^grid_refinement):N_master_mean),grid_refinement)  = 1;
var_grid_index((1 :(2^grid_level/2^grid_refinement):N_master_var),grid_refinement)   = 1;
GRID_MATRIX(:,:,grid_refinement) =  mean_grid_index(:,grid_refinement)*var_grid_index(:,grid_refinement)';
end
CHECK_now  = GRID_MATRIX(:,:,1);
CHECK_now  = CHECK_now(:);
THETA_pass = [];

%% Andrews-Shi
number_of_thetas_checked = 0;
for grid_refinement = 1:grid_level
    disp(grid_refinement)

alpha_smart = MASTER_BETA(CHECK_now>0,1);
beta_smart  = MASTER_BETA(CHECK_now>0,2);

beta_smart= beta_smart(alpha_smart>0);
alpha_smart= alpha_smart(alpha_smart>0);

theta_grid(:,1)    = alpha_smart(:);
theta_grid(:,2)    = beta_smart(:);
number_of_thetas_checked = number_of_thetas_checked + length(theta_grid);

c_vals       = zeros(size(theta_grid,1),1);
CV           = zeros(size(theta_grid,1),1);
parfor theta_index=1:size(theta_grid,1)
    
    temp_theta                      = theta_grid(theta_index,:);
     mtheta_jt_temp                 = moments_theta_logit(temp_theta,TT_r,Logit_freq_by_type_and_r,ind_type_t_in_j);
    g_ineq_temp                     = mtheta_jt_temp'*tau_t;
    
    temp                            = mtheta_jt_temp'*tau_tb;
    tau_mjt_squared                 = (mtheta_jt_temp.^2)'*tau_t;
    phi_jt_mjt                      = sum(phi_jt'.*mtheta_jt_temp,1);    
    f_stdev_temp                    = (f+tau_mjt_squared-(2/Sample_Size)*phi_jt_mjt') -((f-g_ineq_temp).^2);  
    mtheta_jt_temp                  = [];
    tau_mjt_squared                 = [];
    phi_jt_mjt                      = [];     
    f_stdev_temp(f_stdev_temp<=0)   = 10^(-12);
    f_stdev_temp                    = (f_stdev_temp).^(.5);
    
    m_theta                         = sqrt(Sample_Size)*((f - g_ineq_temp)./f_stdev_temp); % Moment
    m_theta(f_keep == 0,:)          = [];
   
    G_temp                          = sqrt(Sample_Size)./f_stdev_temp.*((f_b-temp)-(f-g_ineq_temp)); 
    G_temp(f_keep==0,:)             = [];
    xi                              = (1./kappa).*m_theta; 
    phi_test                        = KMS_AUX1_phifunc_AS2(xi,Bn_param);   
    S_3_boot                        = max(0,max(G_temp+repmat(phi_test,1,B)));
    c_vals(theta_index)             = quantile(S_3_boot,1-alpha_adj) + eta; 
    CV(theta_index)                 = sum(max(0,m_theta-c_vals(theta_index)).^2); 
end

theta_pass  = theta_grid(CV==0,:);
c_vals_pass = c_vals(CV==0);
THETA_pass  = [THETA_pass
               theta_pass]; %#ok<AGROW>
           
MASTER_pass    = MASTER_pass   +(min(abs(master_alpha(:)-theta_pass(:,1)')+abs(master_beta(:)-theta_pass(:,2)'),[],2)<1e-8);
MASTER_checked = MASTER_checked+(min(abs(master_alpha(:)-theta_grid(:,1)')+abs(master_beta(:)-theta_grid(:,2)'),[],2)<1e-8);

MASTER_Matrix_Pass(MASTER_pass==1)=1;
MASTER_Matrix_Checked(MASTER_checked==1)=1;

for ii=1:N_master_mean
    for jj=1:N_master_var
       if  MASTER_Matrix_Pass(ii,jj)==1
       MASTER_Matrix_Check((max(1,ii-grid_level-1):1:min(ii+grid_level+1,N_master_mean)),(max(1,jj-grid_level-1):1: min(jj+grid_level+1,N_master_var)))=1;
       end
    end
end

MASTER_Matrix_Check     = MASTER_Matrix_Check-MASTER_Matrix_Check.*MASTER_Matrix_Checked;

if grid_refinement<grid_level
MASTER_Matrix_Check_now = MASTER_Matrix_Check.*GRID_MATRIX(:,:,grid_refinement+1);
CHECK_now               = MASTER_Matrix_Check_now(:);
end

clear theta_grid
end

MASTER_Matrix_Pass(MASTER_pass==1)=1;
THETA_pass_Check = MASTER_BETA(MASTER_Matrix_Pass(:)>0,:);  
mean_pass = THETA_pass(:,1)./(THETA_pass(:,1)+THETA_pass(:,2));
var_pass  = THETA_pass(:,1).*THETA_pass(:,2)./(THETA_pass(:,1)+THETA_pass(:,2)).^2./(THETA_pass(:,1)+THETA_pass(:,2)+1);
theta_pass = THETA_pass;
% Save Results:
savefile = ['./Replication_Results/AS_grid_',covariate_group,'_64_2','_LogitFactor',num2str(Logit_Factor),'.mat'];
save(savefile,'theta_pass','Sample_Size','r_bar','alpha','ind_type_t_in_j');
