clear variables
close all

load .\Data\Replication_Data
r_bar                   = .03; 
covariate_group         = 'all';
binning_level           = 8;
dim_p                   = 3; % Set dimension of problem: 2 for parameters of beta distribution, 3 for the shares of choice set size
dim3_level              = 2; % If running on dimension 3, which one: 1=full, 2=full-1, 3=full-2
seed                    = 0; 
B                       = 1000; % bootstrap
bin_thresh              = 100; % minimum size for a bin

% Rationalizability:
Nods                    = 3000; 
NrL                     = 0;
NrH                     = 1;
TT_r                    = linspace(NrL,NrH,Nods);
MM_r                    = length(TT_r);
TT_r_expec              = [0 .5*(TT_r(1:end-1)+TT_r(2:end))];

[first_best, second_best, third_best]   = construct_FST_best(lambda_coll,pp_coll,deduc_list_coll,TT_r_expec*r_bar);
fst_best                                = (first_best+second_best+third_best>0);
f_best                                  = first_best>0;
keep                                    = sum(max(fst_best,[],3).*dd_coll,2)==1;
sum(keep)
f_best_ded                              = squeeze(f_best(:,1,:))*100+squeeze(f_best(:,2,:))*200+squeeze(f_best(:,3,:))*250+squeeze(f_best(:,4,:))*500+squeeze(f_best(:,5,:))*1000;
f_best_ded                              = f_best_ded(keep==1,:);
clear fst_best first_best second_best third_best f_best

% Stats 
ind_200_fbest = f_best_ded==200;
find(max(ind_200_fbest,[],1),1,'last')
r_bar*TT_r(find(max(ind_200_fbest,[],1),1,'last'))
find(max(ind_200_fbest,[],1),1,'first')
r_bar*TT_r(find(max(ind_200_fbest,[],1),1,'first'))
sum(sum(ind_200_fbest))
mean(mean(ind_200_fbest))

lambda_coll                             = lambda_coll(keep==1);
pp_coll                                 = pp_coll(keep==1,:);
dd_coll                                 = dd_coll(keep==1,:);
peter                                   = peter(keep==1,:);
female1                                 = female1(keep==1,:);
drv_age1                                = drv_age1(keep==1,:);

% Covariate Groups:
young_cutoff            = prctile(drv_age1,33);
old_cutoff              = prctile(drv_age1,66);
low_cutoff              = prctile(peter,33);
high_cutoff             = prctile(peter,66);
indicator_young         = drv_age1<=young_cutoff;
indicator_old           = drv_age1>=old_cutoff;
indicator_low         	= peter<=low_cutoff;
indicator_high          = peter>=high_cutoff;

% Indicator group:
if strcmp(covariate_group,'all') 
    master_ind          = ones(size(lambda_coll));
elseif strcmp(covariate_group,'male') 
    master_ind          = female1==0;
elseif strcmp(covariate_group,'female') 
    master_ind          = female1==1;
elseif strcmp(covariate_group,'old') 
    master_ind          = indicator_old==1;
elseif strcmp(covariate_group,'young') 
    master_ind          = indicator_young==1;
elseif strcmp(covariate_group,'low') 
    master_ind          = indicator_low==1;
elseif strcmp(covariate_group,'high') 
    master_ind          = indicator_high==1;
end
lambda_coll             = lambda_coll(master_ind==1);
pp_coll                 = pp_coll(master_ind==1,:);
dd_coll                 = dd_coll(master_ind==1,:);
peter                   = peter(master_ind==1,:);
female1                 = female1(master_ind==1,:);
drv_age1                = drv_age1(master_ind==1,:);

% Construct Types
unique_lambda           = unique(lambda_coll);
unique_prices           = unique(pp_coll,'rows');
temp_price              = repmat(unique_prices,length(unique_lambda),1);
temp_lambda             = repmat(unique_lambda,1,size(unique_prices,1))';
temp_lambda_2           = temp_lambda(:);
unique_pairs            = [temp_price temp_lambda_2];

type_ind                = zeros(length(lambda_coll),1);
parfor ii=1:length(lambda_coll)
    ii
   temp                 = [pp_coll(ii,:) lambda_coll(ii,:)]; 
   type_ind(ii)         = find(ismember(unique_pairs,temp,'rows'));   
end

unique_pairs            = [unique_pairs zeros(size(unique_pairs,1),1)];
parfor tt=1:length(unique_pairs)
    tt
   temp_ind             = type_ind==tt;
   unique_pairs(tt,7)   = sum(temp_ind); 
end

unique_types                    = unique(type_ind);
types_index                     = (1:length(unique_types))';
types_ind                       = zeros(size(type_ind));
types_prices_mus                = zeros(length(unique_types),6);
types_counts_taus               = zeros(length(unique_types),2);
types_choice_counts             = zeros(length(unique_types),5);
for tt=1:length(unique_types)
   temp_type                    = unique_types(tt);
   ind_hhs                      = type_ind==temp_type;
   types_prices_mus(tt,:)       = unique_pairs(temp_type,1:6);
   types_counts_taus(tt,:)      = [unique_pairs(temp_type,7) unique_pairs(temp_type,7)/sum(master_ind)];
   types_choice_counts(tt,:)    = sum(dd_coll(ind_hhs==1,:),1); 
   types_ind(ind_hhs)           = tt;   
end

% Binning Types:
raw                 = [pp_coll(:,5) lambda_coll];
cov_mat             = cov(raw);
transformed         = raw/(chol(cov_mat));

Bin_dtp             = binning_level; % Define number of bins in p dimension
Bin_dtl             = binning_level; % Define number of bins in claim rate dimension 

bin_size            = zeros(Bin_dtp*Bin_dtl,1); 
bin_size_sq         = zeros(Bin_dtp,Bin_dtl  );                     
tp_threshold        = [min(transformed(:,1))-10 quantile(transformed(:,1), 1/Bin_dtp:1/Bin_dtp: 1)];
tl_threshold        = [min(transformed(:,2))-10 quantile(transformed(:,2), 1/Bin_dtl:1/Bin_dtl: 1)];
Bin_dtp             = Bin_dtp+1; 
Bin_dtl             = Bin_dtl+1;   

bin_index_new           = zeros(length(transformed),1);
for jtp = 2: Bin_dtp
    temp_check(jtp-1) = sum(transformed(:,1)<=tp_threshold(jtp) & transformed(:,1)> tp_threshold(jtp-1)); %#ok<SAGROW>
    for jtl = 2:Bin_dtl
        temp = transformed(:,1)<=tp_threshold(jtp) & transformed(:,1)> tp_threshold(jtp-1) & transformed(:,2)<=tl_threshold(jtl) & transformed(:,2)> tl_threshold(jtl-1);
        bin_index_new                              = bin_index_new+temp*((jtp-2)*(Bin_dtl-1)+(jtl-2)+1);
        bin_size((jtp-2)*(Bin_dtl-1)+(jtl-2)+1)= sum(temp);
        bin_size_sq(jtp-1,jtl-1)= sum(temp);
    end
end 

temp_bin_type_new   = [types_ind bin_index_new];
map_type_bin_new    = unique(temp_bin_type_new,'rows');
bin_ind             = bin_index_new;
map_type_bin        = map_type_bin_new;
individ_data        = [types_ind dd_coll];

bin_N = binning_level^2;
savefile = ['.\Replication_IntermediateData\data_temp_',covariate_group,'_',num2str(bin_N),'.mat'];     
save(savefile,'deduc_list_coll','r_bar','TT_r','TT_r_expec','MM_r','map_type_bin','bin_size','individ_data','types_index','types_prices_mus','types_counts_taus','types_choice_counts');

if binning_level==8 && strcmp(covariate_group,'all')
    savefile =[ '.\Replication_IntermediateData\data_temp_sumstats_',num2str(bin_N),'.mat'];    
    save(savefile,'deduc_list_coll','r_bar','pp_coll','lambda_coll','dd_coll','individ_data','female1','drv_age1','peter','bin_size','indicator_young','indicator_old','indicator_low','indicator_high');
end


%% Empirical Choice Shares
Sample_Size             = sum(types_counts_taus(:,1));

% Depending on what estimating, set the collection of sets to test
% inequalities
if dim_p==2
    d_sets  = [ 100  0; ...
            200  0; ...
            500  0; ...
            1000 0; ...
            100  200; ...
            500  1000];
elseif dim_p==3       
    d_sets_full  = [ ...
        100  0    0    0; ...
        200  0    0    0; ...
        250  0    0    0; ...
        500  0    0    0; ...
        1000 0    0    0; ...
        100  200  0    0; ...
        100  250  0    0; ...
        200  250  0    0; ...
        200  500  0    0; ...
        250  500  0    0; ...
        250  1000 0    0; ...
        500  1000 0    0;...
        100  200  250  500;...
        100  200  250  1000;...
        100  200  500  1000;...
        100  250  500  1000;...
        200  250  500  1000];     
    d_sets_fullminus1  = [ ...
        100  0    0    0; ...
        200  0    0    0; ...
        250  0    0    0; ...
        500  0    0    0; ...
        1000 0    0    0; ...
        100  200  0    0; ...
        100  250  0    0; ...
        200  250  0    0; ...
        200  500  0    0; ...
        250  500  0    0; ...
        250  1000 0    0; ...
        500  1000 0    0;...
        100  200  250  0;...
        200  500  1000 0;...
        250  500  1000 0];                    
    d_sets_fullminus2  = [  ...
        100  0; ...
        200  0; ...
        500  0; ...
        1000 0; ...
        100  200; ...
        500  1000];                    
    if dim3_level==1 % in this case, pi*f_best + (1-pi)*fst_best.
        d_sets = d_sets_full;        
    elseif dim3_level==2 || dim3_level==3 % for pi2: pi*fs_best + (1-pi)*fst_best; for pi3: pi*fst_best + (1-pi)*fs_best.
        d_sets = d_sets_fullminus1; 
    end    
end
        
% Compute empirical probabilities and counts:
[f,f_keep,J1,phi_jt]    = moments_w( types_choice_counts,types_counts_taus,map_type_bin,d_sets,deduc_list_coll,bin_size,bin_thresh);

% Bootstrap
f_b     = zeros(J1,B);
N_tb    = zeros(size(map_type_bin,1),B);
parfor bb = 1:B
    bb
    stream=RandStream('mlfg6331_64','Seed',seed);
    RandStream.setGlobalStream(stream);
    stream.Substream = bb + B*10^3;

    ind_bb      = ceil(Sample_Size*rand(Sample_Size,1)); % draw sample
    ind_data_bb = individ_data(ind_bb,:); %#ok<PFBNS>
    
    types_choices_bb        = zeros(size(types_choice_counts));
    types_counts_taus_bb    = zeros(size(types_counts_taus));
    for tt=1:size(types_choices_bb,1)
        temp_bb = ind_data_bb(:,1)==tt;
        types_counts_taus_bb(tt,1) = sum(temp_bb);    
        types_choices_bb(tt,:) = sum(ind_data_bb(temp_bb==1,2:end),1);
        types_counts_taus_bb(tt,2) = types_counts_taus_bb(tt,1)/Sample_Size;
    end
    N_tb(:,bb) = types_counts_taus_bb(:,1);
    
    bin_size_bb = zeros(length(bin_size),1);
    for aa=1:length(bin_size)
        temp_bin_ind    = map_type_bin(map_type_bin(:,2)==aa,1); %#ok<PFBNS>
        bin_size_bb(aa)  = sum( types_counts_taus_bb(temp_bin_ind,1));   
    end
    f_temp      = moments_w(types_choices_bb,types_counts_taus_bb,map_type_bin,d_sets,deduc_list_coll,bin_size_bb,bin_thresh);
    f_b(:, bb)  = f_temp;
end

% Save File
bin_N = binning_level^2;
if dim_p==3
    savefile = ['.\Replication_IntermediateData\data_temp_empshares_',covariate_group,'_',num2str(bin_N),'_',num2str(dim_p),'_',num2str(dim3_level),'.mat'];     
else
    savefile = ['.\Replication_IntermediateData\data_temp_empshares_',covariate_group,'_',num2str(bin_N),'_',num2str(dim_p),'.mat'];
end
save(savefile,'f','f_keep','phi_jt','f_b','N_tb','d_sets');


%% Compute Model Implied Shares:
% Fine grid of FST Best for each unique type to get cutoffs:
[first_best, second_best, third_best]   = construct_FST_best(types_prices_mus(:,6),types_prices_mus(:,1:5),deduc_list_coll,TT_r_expec*r_bar);
f_best                                  = (first_best)>0;
fs_best                                 = (first_best+second_best)>0;
fst_best                                = (first_best+second_best+third_best)>0;

% Indicators for LB and UB of risk aversion support where FST_best
% intersects with each K set:
LB_RA_FSTintersect                      = zeros(size(types_index,1),size(d_sets,1));
UB_RA_FSTintersect                      = zeros(size(types_index,1),size(d_sets,1));
LB_RA_FSTintersect_2                    = zeros(size(types_index,1),size(d_sets,1));
UB_RA_FSTintersect_2                    = zeros(size(types_index,1),size(d_sets,1));

for tt=1:size(types_index,1)
    tt
    for dd=1:size(d_sets,1)
        temp_deds           = ismember(deduc_list_coll,d_sets(dd,:));
        temp_grid           = squeeze(max(fst_best(tt,temp_deds==1,:),[],2));
        ind_change_count    = sum(abs(temp_grid(2:end)-temp_grid(1:end-1)));
        if ind_change_count>2
            disp('multiple changes')
            keyboard
        elseif temp_grid(1)==1 && temp_grid(end)==1 && temp_grid(2)==0
            disp('multiple changes - check')
            keyboard            
        elseif (ind_change_count==2 && temp_grid(2)==1 && temp_grid(end)==1) % Check for disjoint regions
            UB_RA_FSTintersect_2 = TT_r(find(temp_grid==0,1,'first')-1);
            LB_RA_FSTintersect_2 = TT_r(find(temp_grid==0,1,'last')+1);
        end
        if max(temp_grid)==1 
            LB_RA_FSTintersect(tt,dd) = TT_r(find(temp_grid==1,1,'first'));
            UB_RA_FSTintersect(tt,dd) = TT_r(find(temp_grid==1,1,'last'));
        end
    end
end


if dim_p==3
    if dim3_level==1
        test_matrix = f_best;
    elseif dim3_level==2 || dim3_level==3 
        test_matrix = fs_best;
    end                    
    LB_RA_FSTintersect_pi   = zeros(size(types_index,1),size(d_sets,1));
    UB_RA_FSTintersect_pi   = zeros(size(types_index,1),size(d_sets,1));
    LB_RA_FSTintersect_pi_2 = zeros(size(types_index,1),size(d_sets,1));
    UB_RA_FSTintersect_pi_2 = zeros(size(types_index,1),size(d_sets,1));
    for tt=1:size(types_index,1)
        tt
        for dd=1:size(d_sets,1)
            temp_deds           = ismember(deduc_list_coll,d_sets(dd,:));
            temp_grid           = squeeze(max(test_matrix(tt,temp_deds==1,:),[],2)); 
            ind_change_count    = sum(abs(temp_grid(2:end)-temp_grid(1:end-1)));
            ind_changes = abs(temp_grid(2:end)-temp_grid(1:end-1));
            if temp_grid(1)==1 && temp_grid(end)==1 && temp_grid(2)==0
                disp('multiple changes - check')
                keyboard 
            elseif ind_change_count==2 
                if temp_grid(2)==1 && temp_grid(end)==1 
                    UB_RA_FSTintersect_pi_2(tt,dd) = TT_r(find(temp_grid==0,1,'first')-1);
                    LB_RA_FSTintersect_pi_2(tt,dd) = TT_r(find(temp_grid==0,1,'last')+1);
                end
            elseif ind_change_count==3
                if temp_grid(1)==0 && temp_grid(end)==1
                    ind_find = find(ind_changes==1);
                    UB_RA_FSTintersect_pi_2(tt,dd) = TT_r(ind_find(2));
                    LB_RA_FSTintersect_pi_2(tt,dd) = TT_r(ind_find(3)+1);
                elseif temp_grid(1)==1 && temp_grid(end)==0 
                    ind_find = find(ind_changes==1);
                    UB_RA_FSTintersect_pi_2(tt,dd) = TT_r(ind_find(1));
                    LB_RA_FSTintersect_pi_2(tt,dd) = TT_r(ind_find(2)+1);
                else
                    disp('check')
                    keyboard
                end
            elseif ind_change_count==4
                if temp_grid(1)==0 && temp_grid(end)==0
                    ind_find = find(ind_changes==1);
                    UB_RA_FSTintersect_pi_2(tt,dd) = TT_r(ind_find(2));
                    LB_RA_FSTintersect_pi_2(tt,dd) = TT_r(ind_find(3)+1);
                else
                    disp('check')
                    keyboard
                end
            elseif ind_change_count>4
                disp('check')
                keyboard
            end
            if max(temp_grid)==1 
                LB_RA_FSTintersect_pi(tt,dd) = TT_r(find(temp_grid==1,1,'first'));
                UB_RA_FSTintersect_pi(tt,dd) = TT_r(find(temp_grid==1,1,'last'));
            end
        end
    end
end

if dim_p==3
    savefile = ['.\Replication_IntermediateData\data_temp_modshares_',covariate_group,'_',num2str(bin_N),'_',num2str(dim_p),'_',num2str(dim3_level),'.mat'];
    save(savefile,'LB_RA_FSTintersect','UB_RA_FSTintersect','LB_RA_FSTintersect_2','UB_RA_FSTintersect_2','d_sets','LB_RA_FSTintersect_pi','UB_RA_FSTintersect_pi','LB_RA_FSTintersect_pi_2','UB_RA_FSTintersect_pi_2'); 
else
    savefile = ['.\Replication_IntermediateData\data_temp_modshares_',covariate_group,'_',num2str(bin_N),'_',num2str(dim_p),'.mat'];
    save(savefile,'LB_RA_FSTintersect','UB_RA_FSTintersect','LB_RA_FSTintersect_2','UB_RA_FSTintersect_2','d_sets'); 
end
