clear variables
close all

covariate_group         = 'high';
ind_binning             = 8;
dim_p                   = 2;

% Load Data
bin_N                   = ind_binning^2;
loadfilename            = ['./Replication_IntermediateData/data_temp_',covariate_group,'_',num2str(bin_N),'.mat'];    
load(loadfilename)
loadfilename            = ['./Replication_IntermediateData/data_temp_empshares_',covariate_group,'_',num2str(bin_N),'_',num2str(dim_p),'.mat'];
load(loadfilename)
loadfilename            = ['./Replication_IntermediateData/data_temp_modshares_',covariate_group,'_',num2str(bin_N),'_',num2str(dim_p),'.mat'];
load(loadfilename)

Sample_Size = size(individ_data,1);
N_j         = size(f,1);
N_t         = size(map_type_bin,1);
N_d         = size(d_sets,1);

% Set parameters for Andrews & Shi:
alpha                   =.05;
eta                     = 10^-6; % Adjustment param for Andrews and Shi, rec 10^-6;
kappa                   = sqrt(.3*log(Sample_Size)); 
Bn_param                = sqrt(.4*log(Sample_Size)/log(log(Sample_Size)));

% Grid parameter space
MEAN_beta               = .0005:.0005:1-.0005;
VAR_beta                = .0005:.0005:.25;
alpha_grid              = (MEAN_beta.^3.*(1./MEAN_beta-1))'*(1./VAR_beta)-repmat(MEAN_beta',1,length(VAR_beta));
beta_grid               = alpha_grid.*repmat(((1-MEAN_beta)./MEAN_beta)',1,length(VAR_beta));
beta_grid               = beta_grid(alpha_grid>0);
alpha_grid              = alpha_grid(alpha_grid>0);
theta_grid(:,1)         = alpha_grid(:);
theta_grid(:,2)         = beta_grid(:);

% Indicator of whether a given type is included in a given constraint
ind_type_t_in_j                     = zeros(N_t,N_j);
ind_type_t_in_j(:,end-N_d+1:end)    = 1;  % Aggregate bin
for tt=1:N_t
   temp_bin = map_type_bin(tt,2);
   ind_type_t_in_j(tt,(temp_bin-1)*N_d+1:temp_bin*N_d)=1;
end

tau_t                   = types_counts_taus(:,2);
tau_tb                  = N_tb./Sample_Size;      
B                       = size(f_b,2);
c_vals                  = zeros(size(theta_grid,1),1);
CV                      = zeros(size(theta_grid,1),1);

alpha_adj = alpha - eta; 
parfor theta_index=1:size(theta_grid,1)
    disp(theta_index)
    temp_theta                      = theta_grid(theta_index,:);
    mtheta_jt_temp                  = moments_theta(temp_theta,LB_RA_FSTintersect,UB_RA_FSTintersect,LB_RA_FSTintersect_2,UB_RA_FSTintersect_2,ind_type_t_in_j);
    g_ineq_temp                     = mtheta_jt_temp'*tau_t;
    g_ineq                          = - g_ineq_temp;
 
    % Standard deviation:
    tau_mjt_squared                 = (mtheta_jt_temp.^2)'*tau_t;
    phi_jt_mjt                      = sum(phi_jt'.*mtheta_jt_temp,1);
    
    var_temp                        = (f+tau_mjt_squared-(2/Sample_Size)*phi_jt_mjt') -((f-g_ineq_temp).^2);
    var_temp(var_temp<=0)            = 10^(-12);
    f_stdev_temp                    = (var_temp).^(.5);   
    m_theta                         = sqrt(Sample_Size)*((f + g_ineq)./f_stdev_temp); 
    m_theta(f_keep == 0,:)          = [];
    
    % Bootstrap:
    temp                            = mtheta_jt_temp'*tau_tb;   
    G_temp                          = (Sample_Size^(.5)./f_stdev_temp).*((f_b-temp)-(f-g_ineq_temp));     
    G_temp(f_keep==0,:)             = []; 
    
    xi                              = (1./kappa).*m_theta; 
    phi_test                        = KMS_AUX1_phifunc_AS2(xi,Bn_param);   
    S_3_boot                        = max(0,max(G_temp+repmat(phi_test,1,B)));
    c_vals(theta_index)             = quantile(S_3_boot,1-alpha_adj) + eta; 
    
    CV(theta_index)                 = sum(max(0,m_theta-c_vals(theta_index)).^2); 
end
theta_pass                          = theta_grid(CV==0,:);
c_vals_pass                         = c_vals(CV==0);

% Save AS Results:
bin_N = ind_binning^2;
savefile = ['./Replication_Results/AS_grid_',covariate_group,'_',num2str(bin_N),'_2.mat'];
save(savefile,'theta_pass','Sample_Size','r_bar','alpha','c_vals_pass','ind_type_t_in_j');


% keyboard

%% KMS Confidence Interval Estimation:
p_vec                   = 3; % projection vector options:  1 = [ 1 0 ]; 2 = [ 0 1 ]; 3 = mean(\nu);
max_direction           = 1; % =1 for max, =0 for min
Bn_param                = inf; % GMS parameter

% Set options structure:
bcmtoptions.B           = B;
bcmtoptions.alpha       = alpha;
bcmtoptions.eta         = eta;
bcmtoptions.bn_param    = Bn_param;
bcmtoptions.n           = Sample_Size;
bcmtoptions.kappa       = kappa;
bcmtoptions.c_B         = norminv(1-alpha/sum(f_keep));
bcmtoptions.beta        = 0.01; % Max bias.
beta                    = bcmtoptions.beta;
rho                     = -norminv((1-(1-beta).^(1/(dim_p*sum(f_keep))))/2);
bcmtoptions.rho         = rho;
bcmtoptions.r_grid      = TT_r;
bcmtoptions.type        = 'two-sided';
bcmtoptions.c_maxit     = 100;
bcmtoptions.hc_tol      = 1/bcmtoptions.B; % Tolerence level for root-finding algorithm
bcmtoptions.c_tol       = 1e-8; % Tolerence level for root-finding algorithm
bcmtoptions.Brent_tol   = 1e-5; % Tolerence level for root-finding algorithm 
bcmtoptions.CVX_maxit   = 25; 
bcmtoptions.S           = 0;

% Refine AS Grid Points:
theta_grid              = get_KMS_theta_grid(p_vec, max_direction,theta_pass,r_bar,0); 
N_j                     = size(f,1);
N_t                     = size(map_type_bin,1);
N_d                     = size(d_sets,1);
tau_t                   = types_counts_taus(:,2);
tau_tb                  = N_tb./Sample_Size;     

c_Estep                 = zeros(size(theta_grid,1),1);
CV_Estep                = zeros(size(theta_grid,1),1);
parfor tt=1:size(theta_grid,1)
    theta_val                           = theta_grid(tt,:);
    [m_temp,g_grid,Dg_grid,f_stdev,G]   = full_moments_theta(theta_val,LB_RA_FSTintersect,UB_RA_FSTintersect,LB_RA_FSTintersect_2,UB_RA_FSTintersect_2,ind_type_t_in_j,f,f_b,tau_t,tau_tb,phi_jt,Sample_Size);
    [c_Estep(tt,:),CV_Estep(tt,:)]      = get_KMS_critical_value_clean(theta_val,f,f_keep,f_stdev,G,g_grid,Dg_grid,p_vec,Sample_Size,bcmtoptions);    
end 
theta_pass_KMS          = theta_grid(CV_Estep==0,:);
c_pass_KMS              = c_Estep(CV_Estep==0);
mean_KMS_pass           = r_bar.*theta_pass_KMS(:,1)./(theta_pass_KMS(:,1)+theta_pass_KMS(:,2));
var_KMS_pass            = r_bar.^2.*theta_pass_KMS(:,1).*theta_pass_KMS(:,2)./((theta_pass_KMS(:,1)+theta_pass_KMS(:,2)).^2.*(theta_pass_KMS(:,1)+theta_pass_KMS(:,2)+1));

% Save Results:
savefile = ['./Replication_Results/KMS_CI_',covariate_group,'_',num2str(bin_N),'_2_',num2str(p_vec),'_',num2str(max_direction),'.mat'];
save(savefile,'theta_pass_KMS','c_pass_KMS','mean_KMS_pass','var_KMS_pass');


