clear variables
close all

covariate_group         = 'young';
ind_binning             = 8;
dim_p                   = 3;
dim3_level              = 2; % 1=full, 2=full-1, 3=full-2

% Load Data
bin_N                   = ind_binning^2;
loadfilename            = ['./Replication_IntermediateData/data_temp_',covariate_group,'_',num2str(bin_N),'.mat'];    
load(loadfilename)
loadfilename            = ['./Replication_IntermediateData/data_temp_empshares_',covariate_group,'_',num2str(bin_N),'_',num2str(dim_p),'_',num2str(dim3_level),'.mat'];
load(loadfilename)
loadfilename            = ['./Replication_IntermediateData/data_temp_modshares_',covariate_group,'_',num2str(bin_N),'_',num2str(dim_p),'_',num2str(dim3_level),'.mat'];
load(loadfilename)

Sample_Size = size(individ_data,1);
N_j         = size(f,1);
N_t         = size(map_type_bin,1);
N_d         = size(d_sets,1);

% Set parameters for Andrews & Shi:
alpha                   =.05;
eta                     = 10^-6; % Adjustment param for Andrews and Shi - rec 10^-6;
kappa                   = sqrt(.3*log(Sample_Size)); 
Bn_param                = sqrt(.4*log(Sample_Size)/log(log(Sample_Size))); 

% Grid parameter space - initial run, then refined run:
if dim3_level == 1 
    pi_range                =  .1:.1:.4;
elseif dim3_level==3
    pi_range = .1:.1:.2;
else
    pi_range                =  .8:.1:1;
end
if dim3_level==1 && (strcmp(covariate_group,'young') || strcmp(covariate_group,'low'))
    beta_alpha_range            = .01:.75:50   ;
    beta_beta_range             = .01:1:625.01; 
else
    beta_alpha_range            = .01:.25:10   ;
    beta_beta_range             = .01:.25:100.01; 
end

theta_grid_coarse           = zeros(length(beta_alpha_range)*length(beta_beta_range)*length(pi_range),3);
temp                        = repmat(      beta_alpha_range, length(beta_beta_range)*length(pi_range),1);
theta_grid_coarse(:,1)      = temp(:);
temp                        = repmat(beta_beta_range,length(pi_range),1);
theta_grid_coarse(:,2)      = repmat(temp(:),length(beta_alpha_range),1);
theta_grid_coarse(:,3)      = repmat(pi_range',length(beta_alpha_range)*length(beta_beta_range),1);

% Indicator of whether a given type is included in a given constraint
ind_type_t_in_j                     = zeros(N_t,N_j);
ind_type_t_in_j(:,end-N_d+1:end)    = 1;  % Aggregate bin
for tt=1:N_t
   temp_bin                         = map_type_bin(tt,2);
   ind_type_t_in_j(tt,(temp_bin-1)*N_d+1:temp_bin*N_d)=1;
end

tau_t               = types_counts_taus(:,2);
tau_tb              = N_tb./Sample_Size;
B                   = size(f_b,2);
alpha_adj           = alpha - eta;
c_vals_coarse       = zeros(size(theta_grid_coarse,1),1);
CV_coarse           = zeros(size(theta_grid_coarse,1),1);
ind_interior_coarse = zeros(size(theta_grid_coarse,1),1);

parfor theta_index=1:size(theta_grid_coarse,1)
    theta_index
    temp_theta      = theta_grid_coarse(theta_index,:);
    mtheta_jt_temp  = moments_theta_pi(temp_theta,LB_RA_FSTintersect,    UB_RA_FSTintersect,    LB_RA_FSTintersect_2,    UB_RA_FSTintersect_2,...
                                                             LB_RA_FSTintersect_pi,UB_RA_FSTintersect_pi,LB_RA_FSTintersect_pi_2,UB_RA_FSTintersect_pi_2,...
                                                             ind_type_t_in_j,dim3_level);
    g_ineq_temp                     = mtheta_jt_temp'*tau_t;
    g_ineq                          = - g_ineq_temp;      
    % Standard deviation:
    tau_mjt_squared                 = (mtheta_jt_temp.^2)'*tau_t;
    phi_jt_mjt                      = sum(phi_jt'.*mtheta_jt_temp,1);
    var_temp                        = (f+tau_mjt_squared-(2/Sample_Size)*phi_jt_mjt') -((f-g_ineq_temp).^2);
    var_temp(var_temp<=10^(-12))    = 10^(-12); 
    f_stdev_temp                    = (var_temp).^(.5);
    
    m_theta                         = sqrt(Sample_Size)*((f + g_ineq)./f_stdev_temp); 
    m_theta(f_keep == 0,:)          = [];

    temp                            = mtheta_jt_temp'*tau_tb;
    G_temp                          = (Sample_Size^(.5)./f_stdev_temp).*((f_b-temp)-(f-g_ineq_temp)); 
    G_temp(f_keep==0,:)             = [];

    xi                              = (1./kappa).*m_theta;
    phi_test                        = KMS_AUX1_phifunc_AS2(xi,Bn_param);   
    S_3_boot                        = max(0,max(G_temp+repmat(phi_test,1,B)));
    c_vals_coarse(theta_index)      = quantile(S_3_boot,1-alpha_adj) + eta;
    CV_coarse(theta_index)          = sum(max(0,m_theta-c_vals_coarse(theta_index)).^2);
    ind_interior_coarse(theta_index)= sum(max(0,m_theta).^2); 
end
theta_pass_coarse                   = theta_grid_coarse(CV_coarse==0,:);
c_vals_pass_coarse                  = c_vals_coarse(CV_coarse==0);

max(theta_pass_coarse)
min(theta_pass_coarse)

% Refine Theta Grid based on Above:
if strcmp(covariate_group,'young') || strcmp(covariate_group,'low')
    ind_yl = 1;
else
    ind_yl=0;
end
theta_grid                          = refine_AS_grid_dim3(theta_pass_coarse,dim3_level,beta_alpha_range,beta_beta_range,ind_yl);
c_vals                              = zeros(size(theta_grid,1),1);
CV                                  = zeros(size(theta_grid,1),1);
parfor theta_index=1:size(theta_grid,1)
    theta_index
    temp_theta      = theta_grid(theta_index,:);
    mtheta_jt_temp  = moments_theta_pi(temp_theta,LB_RA_FSTintersect,    UB_RA_FSTintersect,    LB_RA_FSTintersect_2,    UB_RA_FSTintersect_2,...
                                                             LB_RA_FSTintersect_pi,UB_RA_FSTintersect_pi,LB_RA_FSTintersect_pi_2,UB_RA_FSTintersect_pi_2,...
                                                             ind_type_t_in_j,dim3_level);
    g_ineq_temp                         = mtheta_jt_temp'*tau_t;
    g_ineq                              = - g_ineq_temp;       
    % Standard deviation:
    tau_mjt_squared                     = (mtheta_jt_temp.^2)'*tau_t;
    phi_jt_mjt                          = sum(phi_jt'.*mtheta_jt_temp,1);
    var_temp                            = (f+tau_mjt_squared-(2/Sample_Size)*phi_jt_mjt') -((f-g_ineq_temp).^2);
    
    var_temp(var_temp<=10^(-12))        = 10^(-12); 
    f_stdev_temp                        = (var_temp).^(.5);
    
    m_theta                             = sqrt(Sample_Size)*((f + g_ineq)./f_stdev_temp); 
    m_theta(f_keep == 0,:)              = [];
    temp                                = mtheta_jt_temp'*tau_tb;
    G_temp                              = (Sample_Size^(.5)./f_stdev_temp).*((f_b-temp)-(f-g_ineq_temp));
    G_temp(f_keep==0,:)                 = [];

    xi                                  = (1./kappa).*m_theta; 
    phi_test                            = KMS_AUX1_phifunc_AS2(xi,Bn_param);   
    S_3_boot                            = max(0,max(G_temp+repmat(phi_test,1,B)));
    c_vals(theta_index)                 = quantile(S_3_boot,1-alpha_adj) + eta;
    CV(theta_index)                     = sum(max(0,m_theta-c_vals(theta_index)).^2);
end
theta_pass                              = theta_grid(CV==0,:);
c_vals_pass                             = c_vals(CV==0);
min(theta_pass)
max(theta_pass)


% Save AS Results:
bin_N = ind_binning^2;
savefile = ['./Replication_Results/AS_grid_',covariate_group,'_',num2str(bin_N),'_3_',num2str(dim3_level),'.mat'];
save(savefile,'theta_pass','Sample_Size','r_bar','alpha','c_vals_pass','ind_type_t_in_j');

% keyboard

%% KMS Confidence Interval Estimation:
p_vec                   = dim3_level+9; % projection vector options: 10=full, 11=full-1, 12=full-2
Bn_param                = inf; % GMS parameter
if (p_vec==10 || p_vec==11) 
    max_direction = 1;
elseif p_vec==12 
    max_direction = 0;
end

% Set options structure:
bcmtoptions.B           = B;
bcmtoptions.alpha       = alpha;
bcmtoptions.eta         = eta;
bcmtoptions.bn_param    = Bn_param;
bcmtoptions.n           = Sample_Size;
bcmtoptions.kappa       = kappa;
bcmtoptions.c_B         = norminv(1-alpha/sum(f_keep));
bcmtoptions.beta        = 0.01; % Max bias.
beta                    = bcmtoptions.beta;
rho                     = -norminv((1-(1-beta).^(1/(dim_p*sum(f_keep))))/2);
bcmtoptions.rho         = rho;
bcmtoptions.r_grid      = TT_r;
bcmtoptions.c_maxit     = 100;
bcmtoptions.hc_tol      = 1/bcmtoptions.B; % Tolerence level for root-finding algorithm
bcmtoptions.c_tol       = 1e-8; % Tolerence level for root-finding algorithm
bcmtoptions.Brent_tol   = 1e-5; % Tolerence level for root-finding algorithm 
bcmtoptions.CVX_maxit   = 25; 
bcmtoptions.S           = 0;
if p_vec == 10 || p_vec==11
    bcmtoptions.type    = 'one-sided-UB';       
elseif p_vec == 12 
    bcmtoptions.type    = 'one-sided-LB';
end

% Refine AS Grid Points:
if dim3_level==2 && (strcmp(covariate_group,'male') || strcmp(covariate_group,'old'))
    ind_bigger_grid=1;
else
    ind_bigger_grid=0;
end    
theta_grid              = get_KMS_theta_grid(p_vec, max_direction,theta_pass,r_bar,ind_bigger_grid); 
N_j                     = size(f,1);
N_t                     = size(map_type_bin,1);
N_d                     = size(d_sets,1);
tau_t                   = types_counts_taus(:,2);
tau_tb                  = N_tb./Sample_Size;     

c_Estep                 = zeros(size(theta_grid,1),1);
CV_Estep                = zeros(size(theta_grid,1),1);
parfor tt=1:size(theta_grid,1)
    theta_val                           = theta_grid(tt,:);
    [m_temp,g_grid,Dg_grid,f_stdev,G]                = full_moments_theta_pi(theta_val,LB_RA_FSTintersect,   UB_RA_FSTintersect,   LB_RA_FSTintersect_2,   UB_RA_FSTintersect_2,...
                                                                                                          LB_RA_FSTintersect_pi,UB_RA_FSTintersect_pi,LB_RA_FSTintersect_pi_2,UB_RA_FSTintersect_pi_2,...
                                                                                                          ind_type_t_in_j,f,f_b,tau_t,tau_tb,phi_jt,Sample_Size,dim3_level);
    [c_Estep(tt,:),CV_Estep(tt,:)]    = get_KMS_critical_value_clean(theta_val,f,f_keep,f_stdev,G,g_grid,Dg_grid,p_vec,Sample_Size,bcmtoptions);    
end 
theta_pass_KMS          = theta_grid(CV_Estep==0,:);
c_pass_KMS              = c_Estep(CV_Estep==0);
min(theta_pass_KMS)
max(theta_pass_KMS)

% Save Results:
savefile = ['./Replication_Results/KMS_CI_',covariate_group,'_',num2str(bin_N),'_3_',num2str(p_vec),'_',num2str(max_direction),'.mat'];
save(savefile,'theta_pass_KMS','c_pass_KMS');


