clear variables
close all
N_menu_List     = [7 12 17];
%% Specify DGP and construct the choice data.
for NN = 1:length(N_menu_List)
Kappa           = 5;      % minimum # of alternatives in the choice set used by the researcher
N_menu          = N_menu_List(NN);   % size of the feasible set
DD_list         = linspace(10,1010,N_menu); % deductibles
PP_mult         = (DD_list(end)-DD_list)./DD_list(end); % price multiplier
p_bar_min       =  10;
p_bar_max       =  1000;
N_p_bar         = 100;
pp_bar          = linspace(p_bar_min,p_bar_max,N_p_bar);
PP_list         = pp_bar'*PP_mult;

lambda                  = .10;
r_bar                   = .01; % upper bar on risk aversion
Nods                    = 121; 
NrL                     = 0;
NrH                     = 1;
TT_r                    = linspace(NrL,NrH,Nods);
MM_r                    = length(TT_r);
TT_r_expec              = [0 .5*(TT_r(1:end-1)+TT_r(2:end))];
beta_1                  = 1;
beta_2                  = 1;
mean_DGP                = beta_1/(beta_1+beta_2)*r_bar;
var_DGP                 = beta_1*beta_2/(beta_1+beta_2)^2/(beta_1+beta_2+1)*r_bar^2;

average_price_diff      = mean(abs(mean(diff(PP_list,[],2))));
std_ExVT1               = pi/sqrt(6);

Logit_Factor            = .10;
Logit_Sigma             = Logit_Factor*abs(average_price_diff)/std_ExVT1;

Weighted_CE             = construct_Weighted_CE(repmat(lambda,N_p_bar,1),PP_list,DD_list,TT_r_expec*r_bar,Logit_Sigma);
Weighted_CE             = Weighted_CE - repmat(max(Weighted_CE,[],2),1,N_menu,1);
Logit_Weighted_CE       = exp(Weighted_CE)./sum(exp(Weighted_CE),2);

%% The first step for logit prob. evaluations is to construct the probablity of each possible
tic
% D^*_{\kappa} set. - these are called DD_Star_Sets; GG_Star_Sets are
% indexed versions of the same sets.
% To do that we will be relying on exculsion/inclusion formula
% This formula will take as input Logit probabilities computed over all
% possible realizations of Sets sized N_menu to kappa
DD = [];
for ii= N_menu:-1: Kappa
    DD = [DD
          nchoosek(1:N_menu,ii) zeros(nchoosek(N_menu,ii),N_menu-ii)]; %#ok<AGROW>
end
DD_Sets = NaN*DD;
GG_Sets = 0*DD;
for ii=1:length(DD)
temp  = DD(ii,:);
temp  = temp(temp>0);
DD_Sets(ii,:) = [DD_list(temp) zeros(1,N_menu-length(temp))];
GG_Sets(ii,temp) =1; 
end

% construct the D^*'s
ii= N_menu-Kappa+1;
DD = [];
DD = [DD 
      nchoosek(1:N_menu,ii) zeros(nchoosek(N_menu,ii),N_menu-ii)]; %#ok<AGROW>

DD_Star_Sets = NaN*DD;
GG_Star_Sets = 0*DD;
for ii=1:length(DD)
temp  = DD(ii,:);
temp  = temp(temp>0);
DD_Star_Sets(ii,:) = [DD_list(temp) zeros(1,N_menu-length(temp))];
GG_Star_Sets(ii,temp) =1; 
end

GG_Star_Complement = GG_Star_Sets==0;
size_GG_Sets = sum(GG_Sets,2);

%% now, for each DD_Star we need to pick from DD_Sets those sets that
% contain the complement of DD_star - they will enter our
% exclusion/inclusion formula with alternating signs, which will depend on
% the size of these sets.
with_sign_GG_Sets_in_GG_Star = zeros(length(GG_Sets),length(GG_Star_Sets));
for ii = 1:length(GG_Sets)
   
with_sign_GG_Sets_in_GG_Star(ii,:) = (GG_Star_Complement*GG_Sets(ii,:)'-sum(GG_Star_Complement,2)==0)*(-1).^(round(N_menu/2)*2-Kappa+sum(GG_Sets(ii,:)));
end

%% we now construct the sets to check; and construct a logical that checks
% with which D^*s these sets intersect...
DD = [];
for ii= 1 : Kappa -1
    DD = [DD
          nchoosek(1:N_menu,ii) zeros(nchoosek(N_menu,ii),N_menu-ii)]; %#ok<AGROW>
end
DD_Sets_to_check = NaN*DD;
GG_Sets_to_check = 0*DD;
for ii=1:length(DD)
temp  = DD(ii,:);
temp  = temp(temp>0);
DD_Sets_to_check(ii,:) = [DD_list(temp) zeros(1,N_menu-length(temp))];
GG_Sets_to_check(ii,temp) =1; 
end

GG_Sets_to_check_intersect_GG_Star_Sets       = zeros(length(GG_Sets_to_check),length(GG_Star_Sets));
for ii = 1:length(GG_Sets_to_check)
GG_Sets_to_check_intersect_GG_Star_Sets(ii,:) = GG_Star_Sets*GG_Sets_to_check(ii,:)'>0;
end

DD_Sets                       = DD_Sets_to_check(:,1:Kappa-1);
%% now for every p_bar and every value of r we need to sum over all DD^*
% that intersect with our GG_Sets_to_check. This is done in three steps:
% first we constuct the weighted logits for each set DD_set relevant for
% computing prob. of DD^*; Then we take those that go into DD^* and then we
% finally add them taking into account their sign.



if N_menu > 15
delete(gcp('nocreate'))
parpool Local12
end
Logit_freq_by_type_and_r = zeros(N_p_bar,length(GG_Sets_to_check),MM_r);


parfor ii = 1:N_p_bar
    disp(ii)
  
    for jj=1:MM_r
    temp                     = Logit_Weighted_CE(ii,:,jj).*GG_Sets; % same size as GG_Sets; Logit probabilities for each set GG_Sets; zeros if not in the set
    temp                     = (temp./sum(temp,2))*GG_Star_Sets';  % first normalize, then assign probability at which they will contribute to GG_Star_Sets
    
    Logit_freq_by_type_and_r(ii,:,jj)      = GG_Sets_to_check_intersect_GG_Star_Sets*(sum(temp.*with_sign_GG_Sets_in_GG_Star))'; 
                                                    % pick out those in each GG_Star and assigns +/- depending on their size then adds over those intersecting with GG_sets_to_check
    end
  
end

Logit_freq_by_type_and_r(Logit_freq_by_type_and_r<0)=1e-12;
N_Sets                      = length(GG_Sets_to_check);
emp_choice_dist             = zeros(N_p_bar,N_menu);
weights                     = [0 diff(betacdf(TT_r,beta_1,beta_2))];
parfor pp = 1:length(pp_bar)    
    emp_choice_dist(pp,:)   = (squeeze(Logit_Weighted_CE(pp,:,:)))*weights';
end
Sample_Size_K               = 1000;
Sample_Size                 = Sample_Size_K*1000; % Desired sample size
Size_per_pp_bar             = Sample_Size/N_p_bar; % the resulting # of observations per price level - assuming each price is associated with equal number of observations
emp_choice_count            = round(emp_choice_dist*Size_per_pp_bar); %  choice count per pp_bar
n_types                     = sum(emp_choice_count,2);            % # of observations per pp_bar  
Sample_Size                 = sum(n_types);                       % Sample Size - will be the same as the desired Sample size, subject to rounding.

%% construct data moments and correponding bootstrap objects
[f_ineq,f_ineq_keep,~,phi_jt] = moments_w_simulate(emp_choice_count,length(pp_bar),Sample_Size,DD_Sets,DD_list);

cum_count              = emp_choice_count';
cum_count              = cumsum(cum_count(:));
cum_count_with_0       = [0; cum_count];

seed                   = 0;
B                      = 1000;
emp_choice_count_BB    = zeros(length(cum_count),B);

% Bootstrap
f_b          = zeros(length(f_ineq) ,B); 
n_types_b    = zeros(length(n_types),B);
parfor bb = 1:B
    cum_count_with_0bb =cum_count_with_0;
    disp(bb)
    stream=RandStream('mlfg6331_64','Seed',seed);
    RandStream.setGlobalStream(stream);
    stream.Substream = bb + B*10^3;
    
    ind_bb                 = ceil(Sample_Size*rand(Sample_Size,1));
    emp_choice_count_bb    = zeros(length(cum_count),1);
    
    for ii=1:length(cum_count)
        emp_choice_count_bb(ii)     = sum(ind_bb>cum_count_with_0bb(ii) & ind_bb<=cum_count_with_0bb(ii+1));
    end   
    emp_choice_count_BB(:,bb) =   emp_choice_count_bb;
    emp_choice_count_bb       =   reshape(emp_choice_count_bb,N_menu,N_p_bar)';
    
    f_temp             = moments_w_simulate(emp_choice_count_bb,length(pp_bar),Sample_Size,DD_Sets,DD_list);
    f_b(:, bb)         = f_temp;
    n_types_b(:,bb)    = sum(emp_choice_count_bb,2);
end
toc
pre_time_spent = toc;

%% Set AS parameters
tic

alpha                   =.05;
eta                     = 10^-6; 
kappa                   = sqrt(.3*log(Sample_Size));
Bn_param                = sqrt(.4*log(Sample_Size)/log(log(Sample_Size))); 
alpha_adj               = alpha - eta;

f                       = f_ineq;
f_keep                  = f_ineq_keep;
clear f_ineq f_ineq_keep emp_choice_count_BB emp_choice_dist emp_choice_count cum_X_best first_best X_best  Weighted_CE Logit_Weighted_CE GG_Sets GG_Sets_to_check_intersect_GG_Star_Sets

tau_t               = n_types/Sample_Size;
tau_tb              = n_types_b/Sample_Size;
N_j                 = length(f);
N_t                 = length(n_types);
N_d                 = size(DD_Sets,1);
N_types             = repmat(n_types,1,N_d)';
N_types             = N_types(:);
N_types             = [N_types; ones(N_d,1)*Sample_Size]; %#ok<AGROW>

ind_type_t_in_j                     = zeros(N_t,N_j);
ind_type_t_in_j(:,end-N_d+1:end)    = 1;  % Aggregate bin
for tt=1:N_t
   ind_type_t_in_j(tt,(tt-1)*N_d+1:tt*N_d)=1;
end

%% set the master grid 
mean_low        = 0;
mean_high       = 1;
var_low         = 0;
var_high        = 0.25;
grid_level      = 4;

N_master_mean   = 100*2^(grid_level-1);
N_master_var    =  25*2^(grid_level-1);
master_means    = linspace(mean_low,mean_high,N_master_mean+1);
master_vars     = linspace(var_low,var_high,N_master_var+1);
master_means    = master_means(2:end);
master_vars     = master_vars(2:end);
master_alpha    = (master_means.^3.*(1./master_means-1))'*(1./master_vars)-repmat(master_means',1,length(master_vars));
master_beta     = master_alpha.*repmat(((1-master_means)./master_means)',1,length(master_vars));

MASTER_BETA             = [master_alpha(:) master_beta(:)];
temp_mean               = repmat(master_means,1,N_master_var);
temp_var                = repmat(master_vars,N_master_mean,1);
MASTER_MEAN_VAR         = [temp_mean(:) temp_var(:)];
MASTER_pass             = zeros(N_master_mean*N_master_var,1);
MASTER_checked          = zeros(N_master_mean*N_master_var,1);

MASTER_Matrix_Pass      =  zeros(N_master_mean,N_master_var);
MASTER_Matrix_Checked   =  zeros(N_master_mean,N_master_var);
MASTER_Matrix_Check     =  zeros(N_master_mean,N_master_var);

GRID_MATRIX             = zeros(N_master_mean,N_master_var,grid_level);
mean_grid_index         = zeros(N_master_mean,grid_level);
var_grid_index          = zeros(N_master_var,grid_level);

for grid_refinement=1:grid_level
mean_grid_index((1:(2^grid_level/2^grid_refinement):N_master_mean),grid_refinement)  = 1;
var_grid_index((1 :(2^grid_level/2^grid_refinement):N_master_var),grid_refinement)   = 1;
GRID_MATRIX(:,:,grid_refinement) =  mean_grid_index(:,grid_refinement)*var_grid_index(:,grid_refinement)';
end
CHECK_now  = GRID_MATRIX(:,:,1);
CHECK_now  = CHECK_now(:);
THETA_pass = [];

%% Andrews-Shi
number_of_thetas_checked = 0;
for grid_refinement = 1:grid_level
    disp(grid_refinement)

alpha_smart = MASTER_BETA(CHECK_now>0,1);
beta_smart  = MASTER_BETA(CHECK_now>0,2);

beta_smart= beta_smart(alpha_smart>0);
alpha_smart= alpha_smart(alpha_smart>0);

theta_grid(:,1)    = alpha_smart(:);
theta_grid(:,2)    = beta_smart(:);
number_of_thetas_checked = number_of_thetas_checked + length(theta_grid);

c_vals       = zeros(size(theta_grid,1),1);
CV           = zeros(size(theta_grid,1),1);
parfor theta_index=1:size(theta_grid,1)
    
    temp_theta                      = theta_grid(theta_index,:);
    mtheta_jt_temp                  = moments_theta_simulate_logit(temp_theta,TT_r,Logit_freq_by_type_and_r,ind_type_t_in_j,N_j,N_t,N_d);
    g_ineq_temp                     = mtheta_jt_temp'*tau_t;
    
    temp                            = mtheta_jt_temp'*tau_tb;
    tau_mjt_squared                 = (mtheta_jt_temp.^2)'*tau_t;
    phi_jt_mjt                      = sum(phi_jt'.*mtheta_jt_temp,1);    
    f_stdev_temp                    = (f+tau_mjt_squared-(2/Sample_Size)*phi_jt_mjt') -((f-g_ineq_temp).^2);  
    mtheta_jt_temp                  = [];
    tau_mjt_squared                 = [];
    phi_jt_mjt                      = [];     
    f_stdev_temp(f_stdev_temp<=0)   = 10^(-12);
    f_stdev_temp                    = (f_stdev_temp).^(.5);
    
    m_theta                         = sqrt(Sample_Size)*((f - g_ineq_temp)./f_stdev_temp); % Moment
    m_theta(f_keep == 0,:)          = [];
   
    G_temp                          = sqrt(Sample_Size)./f_stdev_temp.*((f_b-temp)-(f-g_ineq_temp)); 
    G_temp(f_keep==0,:)             = [];
    xi                              = (1./kappa).*m_theta; 
    phi_test                        = KMS_AUX1_phifunc_AS2(xi,Bn_param);   
    S_3_boot                        = max(0,max(G_temp+repmat(phi_test,1,B)));
    c_vals(theta_index)             = quantile(S_3_boot,1-alpha_adj) + eta; 
    CV(theta_index)                 = sum(max(0,m_theta-c_vals(theta_index)).^2); 
end

theta_pass  = theta_grid(CV==0,:);
c_vals_pass = c_vals(CV==0);
THETA_pass  = [THETA_pass
               theta_pass]; %#ok<AGROW>
           
MASTER_pass    = MASTER_pass   +(min(abs(master_alpha(:)-theta_pass(:,1)')+abs(master_beta(:)-theta_pass(:,2)'),[],2)<1e-8);
MASTER_checked = MASTER_checked+(min(abs(master_alpha(:)-theta_grid(:,1)')+abs(master_beta(:)-theta_grid(:,2)'),[],2)<1e-8);

MASTER_Matrix_Pass(MASTER_pass==1)=1;
MASTER_Matrix_Checked(MASTER_checked==1)=1;

for ii=1:N_master_mean
    for jj=1:N_master_var
       if  MASTER_Matrix_Pass(ii,jj)==1
       MASTER_Matrix_Check((max(1,ii-grid_level-1):1:min(ii+grid_level+1,N_master_mean)),(max(1,jj-grid_level-1):1: min(jj+grid_level+1,N_master_var)))=1;
       end
    end
end

MASTER_Matrix_Check     = MASTER_Matrix_Check-MASTER_Matrix_Check.*MASTER_Matrix_Checked;

if grid_refinement<grid_level
MASTER_Matrix_Check_now = MASTER_Matrix_Check.*GRID_MATRIX(:,:,grid_refinement+1);
CHECK_now               = MASTER_Matrix_Check_now(:);
end

clear theta_grid
end
toc
time_spent = toc;

MASTER_Matrix_Pass(MASTER_pass==1)=1;
THETA_pass_Check = MASTER_BETA(MASTER_Matrix_Pass(:)>0,:);  
mean_pass = THETA_pass(:,1)./(THETA_pass(:,1)+THETA_pass(:,2));
var_pass  = THETA_pass(:,1).*THETA_pass(:,2)./(THETA_pass(:,1)+THETA_pass(:,2)).^2./(THETA_pass(:,1)+THETA_pass(:,2)+1);

savefile = ['.\Replication_Results\Simulations_NbarP',num2str(N_p_bar),'_Nmenu',num2str(N_menu),'_Kappa',num2str(Kappa),'_LogitFac',num2str(Logit_Factor*100),'_SampSzK',num2str(Sample_Size_K),...
               '_GridLev',num2str(grid_level),'.mat'];
save(savefile,'mean_DGP','var_DGP','theta_pass','THETA_pass','THETA_pass_Check','mean_pass','var_pass','Sample_Size','r_bar','alpha','eta','kappa','Bn_param','c_vals_pass','f','f_keep','number_of_thetas_checked','pre_time_spent','time_spent');

end

%% Simulations figure 3:
clear variables
close all
Kappa           = 5;
N_p_bar         = 100;
N_menu_List     = [7 12 17];
grid_level      = 4;
Logit_Factor    = .10;
Sample_Size_K   = 1000;
disp('Table 6.1. Panel B')
disp( 'q  |D|  kappa   Correlation  num_of_ineq pre_process_time execution_time eval_points execution_time_per_eval_point ')

for ii=1:length(N_menu_List)
N_menu           = N_menu_List(ii);
loadfile = ['.\Replication_Results\Simulations_NbarP',num2str(N_p_bar),'_Nmenu',num2str(N_menu),'_Kappa',num2str(Kappa),'_LogitFac',num2str(Logit_Factor*100),'_SampSzK',num2str(Sample_Size_K),...
               '_GridLev',num2str(grid_level),'.mat'];
load(loadfile)
% disp([ 'N_menu  N_menu  N_menu  N_menu ' num2str(N_menu)])
% disp(['pre_time spent     ' num2str(pre_time_spent/3600) '     hours'])
% disp(['number of total inequalities     ' num2str(length(f))])
% disp(['number of thetas_checked     ' num2str(number_of_thetas_checked)])
% disp(['time spent     ' num2str(time_spent/3600) '     hours'])
% disp(['time spent per one theta     ' num2str(time_spent/number_of_thetas_checked) '    secs'])
% 
% 
disp(['NA  ' num2str(N_menu) '  ' num2str(Kappa) '  none ' '  ' num2str(length(f))  '  ' num2str(pre_time_spent/3600,3) '  ' num2str(time_spent/3600,3) '  ' num2str(number_of_thetas_checked)  '  ' num2str(time_spent/number_of_thetas_checked,3) ])
disp('                    ')

theta_pass= THETA_pass;

mean_pass   = theta_pass(:,1)./(theta_pass(:,1)+theta_pass(:,2));
var_pass    = theta_pass(:,1).*theta_pass(:,2)./(theta_pass(:,1)+theta_pass(:,2)).^2./(theta_pass(:,1)+theta_pass(:,2)+1);
Zoom_In     = 1e6;
mean_pass   = round(mean_pass*Zoom_In);
var_pass    = round(var_pass*Zoom_In);
mean_pass   = reshape(mean_pass,1,[]);
var_pass    = reshape(var_pass,1,[]);
mean_temp   = sort(unique(mean_pass));
var_min     = 0*mean_temp;
var_max     = 0*mean_temp;

for mm_temp = 1:length(mean_temp)
    var_min(mm_temp) = min(var_pass(mean_temp(mm_temp)==mean_pass)); %#ok<*SAGROW>
    var_max(mm_temp) = max(var_pass(mean_temp(mm_temp)==mean_pass)); %#ok<*SAGROW>
end
var_temp = sort(unique(var_pass));
mean_min = 0*var_temp;
mean_max = 0*var_temp;
for vv_temp = 1:length(var_temp)    
    mean_min(vv_temp) = min(mean_pass(var_temp(vv_temp)==var_pass)); %#ok<*SAGROW>
    mean_max(vv_temp) = max(mean_pass(var_temp(vv_temp)==var_pass)); %#ok<*SAGROW>
end
if ii==1
figure(2)
else
h=figure(2);
end
plot([mean_max(1) mean_min mean_max(end)]/Zoom_In*r_bar,[var_temp(1) var_temp var_temp(end)]/Zoom_In*r_bar^2,'Color',[1,.5,0]*ii/length(N_menu_List)+[0,0,1]*(1-ii/length(N_menu_List)),'Linewidth',3)
hold on
plot([mean_min(1) mean_max mean_min(end)]/Zoom_In*r_bar,[var_temp(1) var_temp var_temp(end)]/Zoom_In*r_bar^2,'Color',[1,.5,0]*ii/length(N_menu_List)+[0,0,1]*(1-ii/length(N_menu_List)),'Linewidth',3)
plot([mean_DGP mean_DGP],[0  .25*r_bar^2],'r--', [0 r_bar],[var_DGP var_DGP],'r--','linewidth',.25)
axis([0 r_bar 0 .25*r_bar^2])
axis('square')
ax = gca;
ax.XAxis.Exponent = -2;
xlabel('E$(\nu)$','Interpreter','Latex')
mean_pass = theta_pass(:,1)./(theta_pass(:,1)+theta_pass(:,2));
var_pass  = theta_pass(:,1).*theta_pass(:,2)./(theta_pass(:,1)+theta_pass(:,2)).^2./(theta_pass(:,1)+theta_pass(:,2)+1);
if ii==1
figure(3)
else
h=figure(3);
end


subplot(1,length(N_menu_List),ii) 
plot(mean_pass*r_bar,var_pass*r_bar^2,'color',[ 1 1 1]);
hold on
plot(mean_pass*r_bar,var_pass*r_bar^2,'b.');
plot([mean_DGP mean_DGP],[0  .25*r_bar^2],'r--', [0 r_bar],[var_DGP var_DGP],'r--','linewidth',.25)
axis([0 r_bar 0 .25*r_bar^2])
axis('square')
ax = gca;
ax.XAxis.Exponent = -2;

ylabel('Var$(\nu)$','interpreter','Latex')
xlabel('E$(\nu)$','Interpreter','Latex')


if ii==1
    xlabel('E$(\nu)$','Interpreter','Latex')
    text(r_bar/2, -.1*r_bar^2, ['(a) ' '$| \mathcal{D} | = $' num2str(N_menu) ', ' '$\kappa = $' num2str(Kappa)],'Interpreter','Latex','HorizontalAlignment','center','fontsize',10)        
elseif ii== 2
    xlabel('E$(\nu)$','Interpreter','Latex')
    text(r_bar/2, -.1*r_bar^2, ['(b) ' '$| \mathcal{D} | = $' num2str(N_menu) ', ' '$\kappa = $' num2str(Kappa)],'Interpreter','Latex','HorizontalAlignment','center','fontsize',10)        
elseif ii== 3
    xlabel('E$(\nu)$','Interpreter','Latex')
    text(r_bar/2, -.1*r_bar^2, ['(c) ' '$| \mathcal{D} | = $' num2str(N_menu) ', ' '$\kappa = $' num2str(Kappa)],'Interpreter','Latex','HorizontalAlignment','center','fontsize',10)        
end

end

savefig ('./Replication_Figures/Simulations_Figure_3')
% set(gcf, 'Color', 'w');
% export_fig ./Replication_Figures/Simulations_Figure_3.eps -painters -native

