function c_val = compute_critical_value_KMS(phi_test,f_keep,G,Dg,p,dim_p,KMSoptions)

c_B         = KMSoptions.c_B;
B           = KMSoptions.B;
alpha       = KMSoptions.alpha;
c_maxit     = KMSoptions.c_maxit;                         
hc_tol      = KMSoptions.hc_tol;                           
c_tol       = KMSoptions.c_tol;                           
Brent_tol   = KMSoptions.Brent_tol;                       
eta         = KMSoptions.eta;                          
alpha_adj   = alpha-eta;    

ind_phi = find(phi_test > -inf & f_keep == 1);              
if isempty(ind_phi)
   c_val = 0;
   return
end

a       = 0;
b       = c_B;
psi_a   = test_coverage(a,zeros(B,1),ones(B,1),phi_test,f_keep,G,Dg,p,dim_p,KMSoptions);
h_a     = mean(psi_a) - (1-alpha_adj);
if h_a > 0 
   c_val = 0;
   return
end
psi_b   = test_coverage(b,psi_a,ones(B,1),phi_test,f_keep,G,Dg,p,dim_p,KMSoptions);
h_b     = mean(psi_b) - (1-alpha_adj);

if h_b < 0
   for jj = 1:1000
       b = 2*b;
       psi_b   = test_coverage(b,psi_a,ones(B,1),phi_test,f_keep,G,Dg,p,dim_p,KMSoptions);
       h_b     = mean(psi_b) - (1-alpha_adj);
       if h_b > 0
           break;
       end
   end
   if jj == 1000 
      keyboard
      error('Cannot find root.')
   end
end

if abs(h_b) < c_tol
    c_val = b;
    return;
end

c = a;
h_c = h_a;

mflag = 1;
for ii = 1:c_maxit
    if h_a ~= h_c && h_b ~= h_c
        s = a*h_b*h_c/((h_a - h_b)*(h_a - h_c)) ...
            + b*h_a*h_c/((h_b  - h_a)*(h_b - h_c)) ...
            + c*h_a*h_b/((h_c - h_a)*(h_c - h_b));
    else
        s = b - h_b*(b-a)/(h_b -h_a);
    end
    if (s < (3*a+b)/4 || s > b) ...
            || (mflag == 1 && 2*abs(s-b) >= abs(b-c)) ...
            || (mflag == 0 && 2*abs(s-b) >= abs(c-d)) ...
            || (mflag == 1 && Brent_tol > abs(b-c)) ...
            || (mflag == 0 && Brent_tol > abs(c-d))
        s = 0.5*(a+b);
        mflag = 1;
    else
        mflag = 0;
    end
    psi_s = test_coverage(s,psi_a,psi_b,phi_test,f_keep,G,Dg,p,dim_p,KMSoptions);
    h_s = mean(psi_s) - (1-alpha_adj);

    d = c;
    c = b;
    h_c = h_b;

    if h_a*h_s < 0
        b = s;
        psi_b = psi_s;
        h_b = h_s;
    else
        a = s;
        psi_a = psi_s;
        h_a = h_s;
    end
    if abs(h_a) < abs(h_b)
        x = a;
        psi_x = psi_a;
        h_x = h_a;
        a = b;
        psi_a = psi_b;
        h_a = h_b;
        b = x;
        psi_b = psi_x;
        h_b = h_x;
    end
    if (abs(h_b) < hc_tol || abs(b-a) < c_tol) && h_b >= 0
        c_val = b;
        return;
    end
end

if h_b >= 0
    c_val = b;
else
    c_val = a;
end

c_val = c_val + eta;

end
